unit uBeepSound;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, ExtCtrls, portaudio, cTypes;

Const
  FramesPerBuffer = 128;
  SampleRate = 44100;

Type

{ TBeepSound }

 TBeepSound = class
  protected
    fPlaying: Boolean;
    PlayTimer: TTimer;
    SoundIsOn: Boolean;
    SoundFreq: Integer;
    SoundPhase: Double;
    SoundPhaseStep: Double;

  protected //PA-Werte
    Stream : PPaStream;
    Error : TPaError;
  public
    Constructor Create;
    Destructor Destroy;override;
  public
    Procedure Activate;
    Procedure Deactivate;
    Procedure BeepOn(AFreqHz:Integer);
    Procedure BeepOff;
  protected
    Function GetAmplitude: Double; inline;
    Procedure PhaseIncrement; inline;
  protected
    Function CheckError(AError: TPaError;AErrText:string=''):Boolean;
    procedure PlayTimerCall(Sender: TObject);
  public
    Property Active: Boolean read fPlaying;
end;

function PortaudioCallback(inputBuffer:pointer; OutputBuffer:pointer;
                    framesPerBuffer:culong; timeInfo:PPaStreamCallbackTimeInfo;
          statusFlags:TPaStreamCallbackFlags; UserData:pointer ): CInt32;cdecl;

procedure StreamFinished(UserData:pointer);cdecl;

var
  BeepSound: TBeepSound;

IMPLEMENTATION

uses
  Dialogs, Forms;//, unit1;


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Constructor TBeepSound.Create;
begin
  fPlaying:= false;

  PlayTimer:= TTimer.Create(nil);
  PlayTimer.Interval:= 200;
  PlayTimer.OnTimer:=@PlayTimerCall;
  PlayTimer.Enabled:= false;


  Error:= Pa_Initialize;
  if Error <> 0 then ShowMessage('Error on Portaudio initialization');

end;

//---------------------------------------------------------
Destructor TBeepSound.Destroy;
begin
  Pa_Terminate;

  PlayTimer.Free;

  Inherited Destroy;

end;

//---------------------------------------------------------
Procedure TBeepSound.Activate;
begin
  fPlaying:= true;
  SoundIsOn:= False;
  SoundPhase:= 0;
  SoundPhaseStep:= 0;

  Error:= Pa_OpenDefaultStream(Stream,0,2,paFloat32,SampleRate,FramesPerBuffer,
                                   PPaStreamCallback(@PortaudioCallback),self);
  if CheckError(Error,'Pa_OpenDefaultStream') then EXIT;

{  Error:= Pa_SetStreamFinishedCallback(Stream,PPaStreamFinishedCallback(@StreamFinished));
  if CheckError(Error,'Pa_SetStreamFinishedCallback') then EXIT;}

  Error := Pa_StartStream(Stream);
  if CheckError(Error,'Pa_StartStream') then EXIT;

  PlayTimer.Enabled:= true;

  //The callback starts to be called from here on...

end;


//---------------------------------------------------------
Procedure TBeepSound.Deactivate;
begin
  if not fPlaying then EXIT;
  fPlaying:= false;

  Error:= Pa_StopStream(Stream); CheckError(Error,'Pa_StopStream');
  while Pa_IsStreamActive(Stream) = 1 do sleep(100);
  Error:= Pa_CloseStream(Stream); CheckError(Error,'Pa_CloseStream');

end;

//---------------------------------------------------------
Procedure TBeepSound.BeepOn(AFreqHz:Integer);
begin
  if not fPlaying then EXIT;


  if not SoundIsOn then SoundPhase:= 0;

  SoundFreq:= AFreqHz;
  SoundPhaseStep:= 2*pi* SoundFreq /SampleRate;

  SoundIsOn:= true;

end;

//---------------------------------------------------------
Procedure TBeepSound.BeepOff;
begin
  if not fPlaying then EXIT;
  SoundIsOn:= false;

end;

//---------------------------------------------------------
Function TBeepSound.GetAmplitude:Double;
begin
  Result:= Sin(SoundPhase);
end;

//---------------------------------------------------------
Procedure TBeepSound.PhaseIncrement;
begin
  SoundPhase:= SoundPhase + SoundPhaseStep;
  if SoundPhase > 2*pi then SoundPhase:= SoundPhase - 2*Pi;

end;


//---------------------------------------------------------
Function TBeepSound.CheckError(AError:TPaError;AErrText:string):Boolean;
begin
  if AError = paNoError then EXIT(false);
  ShowMessage('Error:Portaudio:'+Pa_GetErrorText(AError)+#10#13+AErrText);
  EXIT(true);
end;

//---------------------------------------------------------
procedure TBeepSound.PlayTimerCall(Sender: TObject);
begin
  if not fPlaying then begin PlayTimer.Enabled:= false; EXIT; end;

  if Pa_IsStreamActive(Stream) = 1 then EXIT;

  ////Stream beendet
  Error:= Pa_StopStream(Stream); CheckError(Error,'Pa_StopStream');
  Error:= Pa_CloseStream(Stream); CheckError(Error,'Pa_CloseStream');
  fPlaying:= false;
  PlayTimer.Enabled:= false;


end;

//---------------------------------------------------------
// The callback function which is called by PA everytime new Data is needed.
// Remember: ALWAYS USE CDECL or your pointers will be messed up!
// Pointers to this function must be castable to PPaStreamCallback:
function PortaudioCallback( {%H-}inputBuffer : pointer;
                         OutputBuffer : pointer;
                         framesPerBuffer : culong;
                         {%H-}timeInfo : PPaStreamCallbackTimeInfo;
                         {%H-}statusFlags : TPaStreamCallbackFlags;
                         UserData : pointer ) : CInt32; cdecl;

var
  OutBuffer : PCFloat;
  i : culong;
  AudioValue: cfloat;
begin
  OutBuffer := PCFloat(OutputBuffer);


  // Fill the buffer...
  for i := 0 to (FramesPerBuffer-1) do begin


    if BeepSound.SoundIsOn then begin
      BeepSound.PhaseIncrement;
      AudioValue:= BeepSound.GetAmplitude;
    end
    else AudioValue:= 0;

    OutBuffer^ := AudioValue;
    Inc(OutBuffer);
    OutBuffer^ := AudioValue;
    Inc(OutBuffer);
  end;//do


  {if EndOfData
  then exit(CInt32(paAbort))
  else exit(CInt32(paContinue));}

  exit(CInt32(paContinue));

end;

//---------------------------------------------------------
// This is called when playback is finished.
// Remember: ALWAYS USE CDECL or your pointers will be messed up!
// Pointers to this function must be castable to PPaStreamFinishedCallback:
procedure StreamFinished(UserData:pointer);cdecl;
begin
{  //Pa_StopStream(TAudioPlayback(UserData).Stream);

  if assigned(TAudioPlayback(UserData).ShowPlaying)
  then TThread.Synchronize(TThread.CurrentThread,TAudioPlayback(UserData).ShowPlaying);}
end;




//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

INITIALIZATION
  BeepSound:= TBeepSound.Create;
FINALIZATION
  BeepSound.Free;

end.


